package org.fsou.namematch.slavonic;

import org.fsou.common.*;

/**
 * Matcher of slavonic first names written in English. 
 * Takes into account prevalent rules of slavonic name transliteration.
 * @author Alexander Yevstafyev
 * @version 1.0.0
 * @see SlavonicSurnameMatcher
 */
public class SlavonicNameMatcher extends SlavonicMatcher {

	// Singleton pattern	
	static private SlavonicNameMatcher matcher = null;

	/**
	 * Gives instance of matcher.
	 * @return Matcher instance
	 */
	static public synchronized SlavonicNameMatcher getInstance() {
		if (matcher==null) {
			matcher = new SlavonicNameMatcher();
		}
		return matcher;
	}

	/**
	 * Prevents direct instantiation of class to implement Singleton design
	 * pattern. 
	 */	
	protected SlavonicNameMatcher() {
	}

	/**
	 * Replaces widespread combinations of several english letters which
	 * corespond to a single slavonic one (but bewares of "ts" replacement).
	 * Also replaces some single consonants to make them identical to their
	 * equivalents (in context of slavonic-english transliteration).
	 */	
	protected String replaceCombinations(String name) {
		name = StringUtil.replace(name, "sch", "1");
		name = StringUtil.replace(name, "sh", "2");
		name = StringUtil.replace(name, "ch", "3");
		name = StringUtil.replace(name, "zh", "4");
		name = StringUtil.replace(name, "kh", "h");
		name = StringUtil.replace(name, "ck", "k");
		name = StringUtil.replace(name, "j", "y");
		name = StringUtil.replace(name, "w", "v");
		name = StringUtil.replace(name, "x", "ks");
		return name;
	}
	
	protected boolean isConsonant(char letter) {
		return super.isConsonant(letter)
		    || letter=='1'
		    || letter=='2'
		    || letter=='3'
		    || letter=='4';
	}
}